#include	<string.h>
#include	<stdlib.h>

#include	"column.h"
#include	"params.h"

/* ---------------------------------------------------------------------
	Set up the default mode, record length etc.
	Default is text mode with maximum line length of 255. Processing
	occurs for all records.
   --------------------------------------------------------------------- */

void	defaults(PARAMS* pParams)
{
	pParams->fQuestion		=	FALSE;
	pParams->fTextMode		=	TRUE;
	pParams->iRecLen			=	255;
	pParams->iStartLine		=	0;
	pParams->iEndLine			=	0;
}

/* ---------------------------------------------------------------------
	Read the command line arguments to determine the record lengths,
	record number range and whether to use text mode.
	TransformSetup does the analysis of how to re-arrange the columns.
   --------------------------------------------------------------------- */

int	ParamsSetup(PARAMS* pParams, int argc, char** argv)
{
	int	i;
	int	iNewRecLen, iMaxCol;
	int	cStart, cEnd, cLength, cNew;
	char	*parm;
	char	chType;

	/*	first determine the new record length after changes */

	iMaxCol = 0;
	iNewRecLen = 0;

	for (i=3; i<argc; i++) {
		parm = strupr(argv[i]);
		switch (*parm++) {
			case 'T'	:
				pParams->fTextMode = TRUE;
				pParams->iRecLen = read_param(&parm, pParams->iRecLen);
				break;

			case 'F'	:
				pParams->fTextMode = FALSE;
				pParams->iRecLen = read_param(&parm, pParams->iRecLen);
				break;

			case 'L'	:
				pParams->iStartLine	= read_param(&parm, pParams->iStartLine);
				pParams->iEndLine		= read_param(&parm, pParams->iEndLine);

				if (pParams->iEndLine - pParams->iStartLine < 0)
					error_handler(6, argv[i]);
				break;

			case 'R'	:
				cStart = read_param(&parm, 1);
				cEnd = read_param(&parm, pParams->iRecLen);

				if ((cLength = 1 + cEnd - cStart) <= 0)
					error_handler(6, argv[i]);

				chType = *parm++;
				cNew = read_param(&parm, 1);

				switch (chType) {
					case 'C'	:
						iNewRecLen += cLength;
						iMaxCol = max(iMaxCol, cNew + cLength - 1);
						break;

					case 'D'	:
						iNewRecLen -= cLength;
						break;

					case 'Z'	:
					case 'B'	:
					case 'S'	:
						iMaxCol = max(iMaxCol, cEnd);
						break;

					case 'M' :
						iMaxCol = max(iMaxCol, cNew + cLength - 1);
						break;

					default	:
						error_handler(5, argv[i]);
						break;
				}
				break;

			case 'Q'	:
				pParams->fQuestion = TRUE;
				break;

			default	:
				error_handler(5, argv[i]);
				break;
		}
	}

	if (pParams->fTextMode)
		pParams->iRecLen += 1;		/* adjust for end of string code */

	iNewRecLen += pParams->iRecLen;
	iNewRecLen = max(iNewRecLen, iMaxCol);

	if (pParams->iRecLen != iNewRecLen &&
		(pParams->iStartLine !=0 || pParams->iEndLine != 0) &&
		!pParams->fTextMode)
		error_handler(7, 0);

	return iNewRecLen;
}

/* ---------------------------------------------------------------------
	Check whether a line number is within a set of inclusive bounds.
	Note that if a bound is zero, it does not apply.
	The arguments are:
		pParams	=	specification of which line numbers are allowed.
		value		=	value to test
	Return value is TRUE if within range and FALSE otherwise.
   --------------------------------------------------------------------- */

BOOL	in_range(PARAMS* pParams, int value)
{
	if (pParams->iStartLine	!= 0 && value < pParams->iStartLine)	return FALSE;
	if (pParams->iEndLine	!= 0 && value > pParams->iEndLine)		return FALSE;
	return TRUE;
}


